// ======================================================================
//
// MenuText.cpp
// jeff grills
//
// copyright 1997 origin systems inc
//
// -qq- would like to make the text edit box take regexps
//
// ======================================================================

#include "First.h"
#include "MenuText.h"

#include "Gl.h"
#include "Iff.h"
#include "StringFile.h"
#include "Mouse.h"
#include "Os.h"

#include <dinput.h>
#include <stdio.h>
#include <stdlib.h>

// ======================================================================

const Tag TAG_FNTS = TAG(F,N,T,S);
const Tag TAG_JSTY = TAG(J,S,T,Y);
const Tag TAG_LANG = TAG(L,A,N,G);
const Tag TAG_LNGS = TAG(L,N,G,S);
const Tag TAG_TBOX = TAG(T,B,O,X);
const Tag TAG_TREL = TAG(T,R,E,L);
const Tag TAG_TXAV = TAG(T,X,A,V);
const Tag TAG_TXEV = TAG(T,X,E,V);
const Tag TAG_TXID = TAG(T,X,I,D);
const Tag TAG_TXEF = TAG(T,X,E,F); // text edit "high light" font
const Tag TAG_TXES = TAG(T,X,E,S); // text edit "high light" shared font
const Tag TAG_TXMX = TAG(T,X,M,X);
const Tag TAG_TXEM = TAG(T,X,E,M); // test edit minumum # of characters (before it accepts it)
const Tag TAG_TXSI = TAG(T,X,S,I); // the shared int i for a text variable (or text int)
const Tag TAG_TXVE = TAG(T,X,V,E); // a text variable entry
const Tag TAG_TXVF = TAG(T,X,V,F); // font for this text variable entry
const Tag TAG_TXVS = TAG(T,X,V,S); // shared font this text variable entry
const Tag TAG_TXVA = TAG(T,X,V,A); // text variable chunk (contains language section and id)
const Tag TAG_ALLW = TAG(A,L,L,W);
const Tag TAG_DALW = TAG(D,A,L,W);

// ======================================================================

MenuText::MenuText(MenuElement *parent, int maxWidth, int maxHeight, Iff *iff)
:	MenuElement(parent, maxWidth, maxHeight, NULL)
{
	font            = NULL;
	justifyRule     = Font::JR_left;
	isTip           = false;
	centerInY       = false;

	load(iff);
}

// ----------------------------------------------------------------------

MenuText::~MenuText(void)
{
	font = NULL;
}

// ----------------------------------------------------------------------

void MenuText::load(Iff *iff)
{
	int             r, g, b;
	char            buffer[128];
	StringFile     *lp;
	int             textId;

	if (iff)
	{
		MenuElement::load(iff);

		if (iff->seekChunk(TAG_TIP, Iff::FlagOptional))
		{
			isTip = true;
			iff->exitChunk();
		}

		if (iff->seekChunk(TAG_FONT, Iff::FlagOptional))
		{
			r = iff->read_int32();
			g = iff->read_int32();
			b = iff->read_int32();
			iff->read_string(buffer, sizeof(buffer));
			font = NEW Font(buffer,r,g,b);
			iff->exitChunk();
		}
		else
		{
			iff->seekChunk(TAG_FNTS);
				iff->read_string(buffer, sizeof(buffer));
				font =  findSharedFont(buffer);
				DEBUG_FATAL(!font, ("MENU_TEXT cannot find shared font '%s'", buffer));
			iff->exitChunk();
		}

		if (iff->seekChunk(TAG_LANG, Iff::FlagOptional))
		{
			iff->read_string(buffer, sizeof(buffer));
			lp = NEW StringFile(buffer);

			textId = iff->read_int32();
			iff->exitChunk();

			string = lp->getStringById(textId);
			delete lp;
			lp=NULL;
		}

		if (iff->seekChunk(TAG_LNGS, Iff::FlagOptional))
		{
			iff->read_string(buffer, sizeof(buffer));
			lp = findSharedLanguage(buffer);

			textId = iff->read_int32();
			iff->exitChunk();

			string = lp->getStringById(textId);
			lp=NULL;
		}

		if (iff->seekChunk(TAG_JSTY, Iff::FlagOptional))
		{
			int temp=iff->read_u_int32();
			switch (temp)
			{
				case 1:
					justifyRule=Font::JR_left;
					break;
				case 2:
					justifyRule=Font::JR_right;
					break;
				case 3:
					justifyRule=Font::JR_centerX;
					break;
				default:
					Fatal("MenuText::load bad justify type (%d)", temp);
			}
			iff->exitChunk(); // 'JSTY'
		}

		textOutputRect = getRect();
		if (iff->seekChunk(TAG_TBOX, Iff::FlagOptional))
		{
			textOutputRect.x0 = getRect().x0 + iff->read_int32();
			textOutputRect.y0 = getRect().y0 + iff->read_int32();
			textOutputRect.x1 = getRect().x0 + iff->read_int32();
			textOutputRect.y1 = getRect().y0 + iff->read_int32();
			iff->exitChunk();
		}
		else
			if (iff->seekChunk(TAG_TREL, Iff::FlagOptional))
			{
				int x, y, w, h;

				x = iff->read_int32();
				y = iff->read_int32();
				w = iff->read_int32();
				h = iff->read_int32();
				iff->exitChunk();

				if (x < 0)
					x += (textOutputRect.x1 - textOutputRect.x0 + 1);
				if (y < 0)
					y += (textOutputRect.y1 - textOutputRect.y0 + 1);

				if (w < 0)
					w += (textOutputRect.x1 - textOutputRect.x0 + 1);
				if (h < 0)
					h += (textOutputRect.y1 - textOutputRect.y0 + 1);

				textOutputRect.x0 = getRect().x0 + x;
				textOutputRect.y0 = getRect().y0 + y;
				textOutputRect.x1 = textOutputRect.x0 + w;
				textOutputRect.y1 = textOutputRect.y0 + h;
			}
	}
}

// ----------------------------------------------------------------------

Font *MenuText::getFont(void) const
{
	return font;
}

// ----------------------------------------------------------------------

const String &MenuText::getTextString(void) const
{
	return string;
}

// ----------------------------------------------------------------------

void MenuText::setFont(Font *newFont)
{
	font = newFont;
}

// ----------------------------------------------------------------------

void  MenuText::setTextString(const String &newString)
{
	string = newString;
}

// ----------------------------------------------------------------------

void MenuText::setTextString(int newTextInt)
{
	char t[64];

	itoa(newTextInt, t, 10);
	string.setNull();
	string.append(t,1);
}

// ----------------------------------------------------------------------

IoResult MenuText::event(IoEventStruct *event)
{
	if (isTip && event->type == IoEventMouseSetCursor && menuRect.isInInclusive(event->mouseX, event->mouseY))
	{
		IoWinManager::setTip(getFont(),getTextString());
		return IoResultBlock;
	}

	return MenuElement::event(event);
}

// ----------------------------------------------------------------------

void MenuText::menuDraw(int x, int y) const
{
	Rect  r;

	if (!isTip)
	{
		r = getTextOutputRect();
		if (centerInY)
			r.y0=(r.y1+r.y0-font->pixelHeight())/2;

		font->printString(getTextString(), r.x0+x, r.y0+y, r.x1+x, justifyRule);
	}
	
	MenuElement::menuDraw(x, y);
}

// ----------------------------------------------------------------------

const Rect& MenuText::getTextOutputRect(void) const
{
	return textOutputRect;
}

// ======================================================================

MenuTextVariable::MenuTextVariable(MenuElement *parent, int maxWidth, int maxHeight, Iff *iff)
: MenuText(parent, maxWidth, maxHeight, NULL)
{
	entry      = NULL;
	numEntries = 0;
	load(iff);
}

// ----------------------------------------------------------------------

MenuTextVariable::~MenuTextVariable(void)
{
	for (int loop=0; loop<numEntries; loop++)
	{
		delete entry[loop].section;
		entry[loop].section=NULL;
	}

	delete []entry;
	entry=NULL;
}

// ----------------------------------------------------------------------

void MenuTextVariable::load(Iff *iff)
{
	const int suggestedMinimumEntries = 2;
	int       i;
	char      buffer[128];
	int       r, g, b;

	if (iff)
	{
		MenuText::load(iff);

		// find the shared integer
		iff->seekChunk(TAG_TXSI);
		iff->read_string(buffer, sizeof(buffer));
		menuResourceInteger = findSharedInteger(buffer);
		DEBUG_FATAL(!menuResourceInteger, ("MENU_TEXT_VARIABLE cannot find shared integer '%s'", buffer));
		iff->exitChunk();

		// load in the entries
		if ((numEntries = iff->blockCount(Iff::FlagForm, TAG_TXVE)) < suggestedMinimumEntries)
			Fatal("MenuTextVar. load must contain at least %d entries to be effective.", suggestedMinimumEntries);
		entry = NEW Entry[numEntries];
		for (i = 0; iff->seekForm(TAG_TXVE, Iff::FlagOptional); i++)
		{
			// does this entry have a font? (default font is contained in MenuText class (our ancestor)
			entry[i].highLight = NULL;
			if (iff->seekChunk(TAG_TXVF, Iff::FlagOptional))
			{
				r = iff->read_int32();
				g = iff->read_int32();
				b = iff->read_int32();
				iff->read_string(buffer, sizeof(buffer));
				entry[i].highLight = NEW Font(buffer,r,g,b);
				iff->exitChunk();
			}
			else
			{
				if (iff->seekChunk(TAG_TXVS, Iff::FlagOptional))
				{
					iff->read_string(buffer, sizeof(buffer));
					entry[i].highLight = findSharedFont(buffer);
					DEBUG_FATAL(!entry[i].highLight, ("MENU_TEXT_VARIABLE cannot find shared font '%s'", buffer));
					iff->exitChunk();
				}
			}

			// read in the text for this entry
			iff->seekChunk(TAG_TXVA);
			iff->read_string(buffer, sizeof(buffer));

			entry[i].section = NEW StringFile(buffer);
			entry[i].textId  = iff->read_int32();

			iff->exitChunk(); // 'TXVA'

			iff->exitForm(); // 'TXVE'
		}
	}
}

// ----------------------------------------------------------------------

const String &MenuTextVariable::getTextString(void) const
{
	int index = menuResourceInteger->getValue();
	return entry[index].section->getStringById(entry[index].textId);
}

// ----------------------------------------------------------------------

Font *MenuTextVariable::getFont(void) const
{
	Font *f;

	// if there isn't a high light font associated with this text string then default to the parent font
	f = entry[menuResourceInteger->getValue()].highLight;
	if (!f)
		f = font;

	return f;
}

// ======================================================================

MenuTextInt::MenuTextInt(MenuElement *parent, int maxWidth, int maxHeight, Iff *iff)
: MenuText(parent, maxWidth, maxHeight, NULL)
{
	menuResourceInteger = NULL;
	load(iff);
}

// ----------------------------------------------------------------------

MenuTextInt::~MenuTextInt(void)
{
}

// ----------------------------------------------------------------------

void MenuTextInt::load(Iff *iff)
{
	char buffer[128];

	if (iff)
	{
		MenuText::load(iff);

		// find the shared integer
		iff->seekChunk(TAG_TXSI);
		iff->read_string(buffer, sizeof(buffer));
		menuResourceInteger = findSharedInteger(buffer);
		DEBUG_FATAL(!menuResourceInteger, ("MENU_TEXT_INT cannot find shared integer '%s'", buffer));
		iff->exitChunk();
	}
}

// ----------------------------------------------------------------------

const String &MenuTextInt::getTextString(void) const
{
	static char dummy[32];

	string.setNull();
	string.append(itoa(menuResourceInteger->getValue(), dummy, 10), 1);
	return string;
}

// ----------------------------------------------------------------------

